unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TForm1 = class(TForm)
    DeviceMemo: TMemo;
    PNPCheck: TCheckBox;
    FirmCheck: TCheckBox;
    SNCheck: TCheckBox;
    UserCheck: TCheckBox;
    Button1: TButton;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB;

function NybbleToHex(N: Byte): Char;
begin
  N := N and $F;
  if N < 10 then
    Result := Char(Byte('0') + N)
  else
    Result := Char(Byte('A') - 10 + N)
  ;
end;

function MBox(Text, Caption: String; Flags: LongWord): LongWord;
begin
  Result := MessageBox(0, PChar(Text), PChar(Caption), Flags);
  Text := '';
  Caption := '';
end;

type
  TAIOUSB_CloseDevice = function (DeviceIndex: LongWord): LongWord; cdecl;

var
  AIOUSB_CloseDevice: TAIOUSB_CloseDevice;

type
  TDeviceData = record
    DI: LongWord;
    Nom: String;
    SN: String;
    EMode: String;
  end;

const
  EMode00 = '00 (erased)';
  EModeC0 = 'C0 (plain)';
  EModeC2 = 'C2 (firmware)';

var
  Dev: TDeviceData;


function SNToStr(SN: Int64): String;
var
  I: Integer;
  
begin
  Result := 'XXXX-XXXX-XXXX-XXXX';
  for I := Length(Result) downto 1 do
    if Result[I] = 'X' then begin
      Result[I] := NybbleToHex(SN);
      SN := SN shr 4;
    end
  ;
end;

function EModeToStr(EMode: Byte): String;
begin
  if EMode = $C0 then
    Result := EModeC0
  else if EMode = $C2 then
    Result := EModeC2
  else if EMode = $00 then
    Result := EMode00
  else
    Result := IntToHex(EMode, 2) + ' (???)'
  ;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
  Status, DI, BufSize: LongWord;
  Buf: String;
  SN: Int64;
  EMode: Byte;

begin
  repeat
    DI := ResolveDeviceIndex(diOnly);
    if DI <> diNone then Break;

    Status := GetLastError;
    case Status of
      ERROR_FILE_NOT_FOUND: if MBox('No ACCES USB devices found. Plug one in to erase it.', 'EClear - No Device Found', MB_RETRYCANCEL) <> IDRETRY then begin
        Application.ShowMainForm := False;
        Application.Terminate;
        Exit;
      end;
      ERROR_DUP_NAME: if MBox('Too many ACCES USB devices found; this program only erases one at a time.', 'EClear - Too Many Devices', MB_RETRYCANCEL) <> IDRETRY then begin
        Application.ShowMainForm := False;
        Application.Terminate;
        Exit;
      end;
      else if MBox('Error ' + IntToStr(Status) + ' detecting ACCES USB devices.', 'EClear - Error ' + IntToStr(Status), MB_RETRYCANCEL) <> IDRETRY then begin
        Application.ShowMainForm := False;
        Application.Terminate;
        Exit;
      end;
    end;
    if Assigned(AIOUSB_CloseDevice) then
      for I := 0 to 31 do AIOUSB_CloseDevice(I)
    ;
    GetDevices;
  until False;

  Dev.DI := DI;

  BufSize := MAX_PATH;
  SetString(Buf, nil, BufSize);
  QueryDeviceInfo(DI, nil, @BufSize, @Buf[1], nil, nil);
  SetLength(Buf, BufSize);
  Dev.Nom := Buf;

  GetDeviceSerialNumber(DI, SN);
  Dev.SN := SNToStr(SN);

  BufSize := 8;
  SetString(Buf, nil, BufSize);
  GenericVendorRead(DI, $A2, $0000, 0, BufSize, @Buf[1]);

  EMode := $FF;
  BufSize := SizeOf(EMode);
  GenericVendorRead(DI, $A2, $0000, 0, BufSize, @EMode);
  Dev.EMode := EModeToStr(EMode);

  DeviceMemo.Lines.Add('Found ' + Dev.Nom);
  DeviceMemo.Lines.Add('SN: ' + Dev.SN);
  DeviceMemo.Lines.Add('EEPROM Mode: ' + Dev.EMode);
  DeviceMemo.Lines.Add('');
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  Close;
end;

procedure EraseEEPROM(Start, Len: Word);
var
  Buf: String;

begin
  Buf := StringOfChar(#$00, Len);
  GenericVendorWrite(Dev.DI, $A2, Start, 0, Length(Buf), @Buf[1]);
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  Buf: String;

begin
  if PNPCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing P&P...');
    EraseEEPROM(0, $8);
    Dev.EMode := EModeToStr($00);
    DeviceMemo.Lines[2] := 'EEPROM Mode: ' + Dev.EMode;
  end;

  if FirmCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing Firmware...');
    EraseEEPROM($8, $1DE8);

    if (Dev.EMode = EModeC2) and not PNPCheck.Checked then begin
      Buf := #$C0;
      GenericVendorWrite(Dev.DI, $A2, $0000, 0, Length(Buf), @Buf[1]);
      Dev.EMode := EModeToStr($C0);
      DeviceMemo.Lines[2] := 'EEPROM Mode: ' + Dev.EMode;
    end;
  end;

  if SNCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing SN & Hashes...');
    EraseEEPROM($1DF0, $10);
    Dev.SN := SNToStr(0);
    DeviceMemo.Lines[1] := 'SN: ' + Dev.SN;
  end;

  if UserCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing User Area...');
    EraseEEPROM($1E00, $200);
  end;
  DeviceMemo.Lines.Add('Done.');
end;





var
  hAIOUSB: THandle;

initialization
  hAIOUSB := LoadLibrary('AIOUSB.dll');
  AIOUSB_CloseDevice := GetProcAddress(hAIOUSB, 'AIOUSB_CloseDevice');
finalization
  FreeLibrary(hAIOUSB);
  hAIOUSB := 0;
end.
